# -*- coding: utf-8 -*-
import sys
PY2 = sys.version_info < (3,)
PY3 = sys.version_info >= (3,)
if PY2:
    from StringIO import StringIO
else:
    from io import StringIO
from os.path import abspath, dirname
from unittest import TestCase

SKOOLKIT_HOME = abspath(dirname(dirname(__file__)))
sys.path.insert(0, SKOOLKIT_HOME)
from skoolkit import bin2tap, skool2asm, skool2ctl, skool2html, skool2sft, sna2skool

class SkoolKitTestCase(TestCase):
    def assert_no_errors(self, error, cmdline, msg=None):
        if msg is None:
            msg = "Error(s) while running '%s':\n-----\n%s\n-----\n" % (cmdline, error)
        self.assertTrue(len(error) == 0, msg)

    def assert_output_equal(self, output, expected, prefixes=True):
        for i, line in enumerate(expected):
            if i < len(output):
                if prefixes:
                    self.assertEqual(output[i][:len(line)], line)
                else:
                    self.assertEqual(output[i], line)
        if len(output) > len(expected):
            self.fail("Unexpected extra line in output: '%s'" % output[len(expected)])
        elif len(expected) > len(output):
            self.fail("Expected lines missing from output, starting with: '%s'" % expected[len(output)])

    def to_lines(self, text, strip_cr):
        # Use rstrip() to remove '\r' characters (useful on Windows)
        if strip_cr:
            lines = [line.rstrip() for line in text.split('\n')]
        else:
            lines = text.split('\n')
        if lines[-1] == '':
            lines.pop()
        return lines

    def _run_skoolkit_command(self, cmd, args, out_lines, err_lines, strip_cr):
        stdout = sys.stdout
        stderr = sys.stderr
        sys.stdout = out = StringIO()
        sys.stderr = err = StringIO()
        try:
            cmd(args.split())
        except SystemExit:
            pass
        out = out.getvalue()
        err = err.getvalue()
        sys.stdout = stdout
        sys.stderr = stderr
        if out_lines:
            out = self.to_lines(out, strip_cr)
        if err_lines:
            err = self.to_lines(err, strip_cr)
        return out, err

    def run_bin2tap(self, args='', out_lines=True, err_lines=False, strip_cr=True):
        return self._run_skoolkit_command(bin2tap.main, args, out_lines, err_lines, strip_cr)

    def run_skool2asm(self, args='', out_lines=True, err_lines=False, strip_cr=True):
        return self._run_skoolkit_command(skool2asm.main, args, out_lines, err_lines, strip_cr)

    def run_skool2ctl(self, args='', out_lines=True, err_lines=False, strip_cr=True):
        return self._run_skoolkit_command(skool2ctl.main, args, out_lines, err_lines, strip_cr)

    def run_skool2html(self, args='', out_lines=True, err_lines=False, strip_cr=True):
        return self._run_skoolkit_command(skool2html.main, args, out_lines, err_lines, strip_cr)

    def run_skool2sft(self, args='', out_lines=True, err_lines=False, strip_cr=True):
        return self._run_skoolkit_command(skool2sft.main, args, out_lines, err_lines, strip_cr)

    def run_sna2skool(self, args='', out_lines=True, err_lines=False, strip_cr=True):
        return self._run_skoolkit_command(sna2skool.main, args, out_lines, err_lines, strip_cr)
